#include <windows.h>
#include <stdio.h>
#include <conio.h>
#include <math.h>
#include "serial.h"
#include "legacy_functions.h"


#define PLUS_KEY       43
#define MINUS_KEY      45
#define ESC_KEY        27
#define NUMBER_OF_AXES 2

#define M_PI			(2*acos(0.0))

#define TOL_CIRCLE		50  	

#define MAX_ROBOT_VEL	4000

#define X_GAIN			1.0
#define THETA_GAIN		0.3

enum serial_cmd
{
	SERIAL_GET_ENCODER = 0,
	SERIAL_SET_PWM,
	SERIAL_GET_ODOMETRY,
	SERIAL_ENB_SERIAL_TIMEOUTS,
	SERIAL_DIS_SERIAL_TIMEOUTS,
	SERIAL_SET_ROBOT_VEL_PC,
	SERIAL_SET_ROBOT_VEL,
	SERIAL_RESET_ODOMETRY,
	SERIAL_SET_TRAP_POS,
	SERIAL_SET_PROP_VEL_MODE
};

enum menu_cmd
{
	MENU_GET_ENCODER = 0,
	MENU_SET_PWM,
	MENU_SET_AXIS,
	MENU_GET_ODOMETRY,
	MENU_SET_VELOCITY,
	MENU_RESET_ODOMETRY,
	MENU_TEST_CLOOP,
	MENU_SET_POS_TRAP,
	MENU_SET_PROP_VEL_MODE,
	MENU_QUIT	
};



unsigned int DoMenu(char **menu)
{
	unsigned int nitems = atoi(menu[0]), i, opt;

	do {
		clrscr();
		for(i = 1; i <= nitems; i++) {
			printf("%d. %s\n", i, menu[i]);
		}
		printf("Select option : ");
		scanf("%d", &opt);
	} while((opt < 1) || (opt > nitems));
	return opt-1;
}


void test_encoder_pwm(unsigned char axis)
{
	int enc;
	char pwm = 0, quit_flag = 0, junk;
	
	clrscr();
	printf("Press + and - keys to increment or decrement PWM. Press ESC to quit\n");
	while (!quit_flag)
	{
		switch(_getch()) {
		case PLUS_KEY:
			if (pwm<100)
				pwm = pwm + 2;
			break;
		case MINUS_KEY:
			if (pwm>-100)
				pwm = pwm - 2;
			break;
		case ESC_KEY:
			quit_flag = 1;
			pwm = 0;
			break;
		default:
			break;
		}
		send_char_serial(SERIAL_SET_PWM);
		receive_char_serial(&junk);
		send_char_serial(axis);
		send_char_serial(pwm);
		receive_char_serial(&junk);

		send_char_serial(SERIAL_GET_ENCODER);
		receive_char_serial(&junk);
		send_char_serial(axis);
		receive_32bit_serial(&enc);
		printf("PWM value = %d \t\t Encoder value = %d\n", pwm, enc);
	}
}


void test_encoder_manual(unsigned char axis)
{
	int enc; char junk;
	
	clrscr();
	while (!kbhit())
	{
		send_char_serial(SERIAL_GET_ENCODER);
		receive_char_serial(&junk);

		send_char_serial(axis);
		receive_32bit_serial(&enc);
		printf("Encoder value = %d\n", enc);
	}
}


int get_actual_pos(unsigned char axis)
{
	int enc; char junk;
	
	send_char_serial(SERIAL_GET_ENCODER);
	receive_char_serial(&junk);
	send_char_serial(axis);
	receive_32bit_serial(&enc);
	return enc;
}


void set_pwm(char pwm, unsigned char axis)
{
	char junk;
	send_char_serial(SERIAL_SET_PWM);
	receive_char_serial(&junk);
	send_char_serial(axis);
	send_char_serial(pwm);
	receive_char_serial(&junk);
}


void test_cloop_pc(void)
{
	unsigned char axis = 0;
	float kp = 0.5, kd = 0.5;
	float posf, pos=0.0, pwm, err, prev_err=0, errd=0;
	FILE *fid;

	fid = fopen("encoder.txt", "w");
	posf = 32000;

	while (!kbhit())
	{
		pos = (float)get_actual_pos(axis);
		fprintf(fid, "%f\n", pos);
		err = (posf - pos);
		errd = err - prev_err;
		prev_err = err;

		pwm = (kp * err) + (kd * errd);
		if (pwm>100.0)
			pwm = 100.0;
		
		if (pwm < -100.0)
			pwm = -100.0;

		//printf("PWM = %d\n", (char)pwm);
		set_pwm( (char)pwm, axis);
	}
	fclose(fid);
}


void get_odometry(int *x, int *y, int *theta)
{

	send_char_serial(SERIAL_GET_ODOMETRY);
	receive_32bit_serial(x);
	receive_32bit_serial(y);
	receive_32bit_serial(theta);
//	printf("X = %d \t Y = %d \t THETA = %f\n", *x, *y, (*theta)*180.0/3124.0);
}


void get_odometry_keyboard(void)
{
	int x, y, theta;
	
	while (!kbhit())
	{
		get_odometry(&x, &y, &theta);
		printf("X = %d \t Y = %d \t THETA = %f\n", x, y, theta*180.0/(M_PI*1000.0));
	}
}


void synchronize_with_robot(void)
{
	const char sync_msg_hh[6] = { 's', 'h', 'r', 'i', 'm', 'p' };
	const char sync_msg_ro[6] = { 't', 'i', 's', 'j', 'n', 'q' };
	char i, rec;
	DWORD status;
	
	set_pc_serial_timeout(50);
	for(i = 0; i < 6; i++) {
		send_char_serial(sync_msg_hh[i]);
		status = receive_char_serial(&rec);
		if(!status)
			i = -1;
		else if(rec != sync_msg_ro[i]) 
		{
			i = -1;
			printf("Error syncing with robot !\nSent %c received %c !!\n", sync_msg_hh[i], sync_msg_ro[i]);
		}
	}
	set_pc_serial_timeout(0);
}


void disable_robot_serial_timeouts(void)
{
	char rec;
	send_char_serial(SERIAL_DIS_SERIAL_TIMEOUTS);
	receive_char_serial(&rec);
}


void enable_robot_serial_timeouts(void)
{
	char rec;
	send_char_serial(SERIAL_ENB_SERIAL_TIMEOUTS);
	receive_char_serial(&rec);
}


void set_robot_velocity(short fb_vel, short rl_vel)
{
	unsigned char junk;
	
	if (fb_vel>MAX_ROBOT_VEL)
		fb_vel=MAX_ROBOT_VEL;
	if (fb_vel<(-1*MAX_ROBOT_VEL))
		fb_vel=-1*MAX_ROBOT_VEL;
	if (rl_vel>MAX_ROBOT_VEL)
		rl_vel=MAX_ROBOT_VEL;
	if (rl_vel<(-1*MAX_ROBOT_VEL))
		rl_vel=-1*MAX_ROBOT_VEL;


	send_char_serial(SERIAL_SET_ROBOT_VEL_PC);
	receive_char_serial(&junk);
	send_16bit_serial(fb_vel);
	Sleep(20);
	send_16bit_serial(rl_vel);
	receive_char_serial(&junk);
//	printf("FB_VELOCITY = %d \t RL_VELOCITY = %d\n", fb_vel, rl_vel);
	
}


void set_robot_velocity_keyboard(void)
{
	short fb_vel = 0, rl_vel = 0;
	unsigned char quit_flag = 0;
	int x, y, theta;

	clrscr();
	printf("Press + and - keys to increment or decrement FWD VELOCITY.\n 'a' & 's' keys for LEFT RIGHT. Press ESC to quit\n");
	while (!quit_flag)
	{
		switch(_getch()) {
		case PLUS_KEY:
			if (fb_vel<MAX_ROBOT_VEL)
				fb_vel = fb_vel + 20;
			break;
		case MINUS_KEY:
			if (fb_vel>-1*MAX_ROBOT_VEL)
				fb_vel = fb_vel - 20;
			break;
		case 'a':	// LEFT
			if (rl_vel<MAX_ROBOT_VEL)
				rl_vel = rl_vel + 5;
			break;
		case 's':	// RIGHT
			if (rl_vel<MAX_ROBOT_VEL)
				rl_vel = rl_vel - 5;
			break;
		case ESC_KEY:
			quit_flag = 1;
			break;
		default:
			break;
		}

		set_robot_velocity(fb_vel, rl_vel);
		get_odometry(&x, &y, &theta);
		printf("X=%6d Y=%6d THETA=%.2f ", x, y, (float)theta * 180.0/(1000.0*M_PI));
		printf("FB_VELOCITY = %d RL_VELOCITY = %d\n", fb_vel, rl_vel);
	}

	while((abs(fb_vel)>10) || (abs(rl_vel)>10))
	{
		fb_vel -= fb_vel/6;
		rl_vel -= rl_vel/2;
		printf("FB_VELOCITY = %d \t RL_VELOCITY = %d\n", fb_vel, rl_vel);
		set_robot_velocity(fb_vel, rl_vel);
	}
	set_robot_velocity(0, 0);

}



void test_odo_closed_loop(void)
{
	int x, y, theta, xf, yf, thetaf, thetaf_prev = 0;
	short fb_vel = 0, rl_vel = 0, n = 0;
	float dist;

	xf = 800;
	yf = 0;
	thetaf = 0;
	
	printf("Type X final:");
	scanf("%d", &xf);
	printf("\nType Y final:");
	scanf("%d", &yf);

	while (!kbhit())
	{
		get_odometry(&x, &y, &theta);
		
		//printf("This is the Odometry x = %d , y = %d , theta = %d ", x , y , theta) ;

		dist = sqrt(((xf-x)*(xf-x)) + ((yf-y)*(yf-y)));
		fb_vel = (short)(X_GAIN*dist);
		thetaf = atan2(yf-y, xf-x) * 1000.0;

//		theta = fmod(theta, (2*M_PI*1000.0));
		if (thetaf<0)
			thetaf = (short)(2.0*M_PI*1000.0) + thetaf;
/*		if (fabs(thetaf - thetaf_prev)>M_PI) {
			if (thetaf > thetaf_prev)
				n -= 1;
			else
				n += 1;
		}
		thetaf += n*2*M_PI;*/
		
		printf("X=%6d Y=%6d THETA=%.2f THETA_F=%.2f \t", x, y, (float)theta * 180.0/(1000.0*M_PI), (float)thetaf * 180.0/(1000.0*M_PI));

		if (dist<TOL_CIRCLE)
			break;


		rl_vel = THETA_GAIN*(thetaf-theta);
		printf("FB=%d \t RL=%d\n", fb_vel, rl_vel);
		set_robot_velocity(fb_vel, rl_vel);
		thetaf_prev = thetaf;
	}
//	_getch();
	set_robot_velocity(0, 0);
	printf("\nPress any key to continue");
	_getch();
}


void set_trap_pos(void)
{
	int x;
	unsigned char junk;

	printf("Type X final:");
	scanf("%d", &x);
	
	send_char_serial(SERIAL_SET_TRAP_POS);
	receive_char_serial(&junk);
	send_16bit_serial((short)x);
	receive_char_serial(&junk);
}


int main(int argc, char* argv[])
{
	unsigned char quit_flag = 0, axis = 1, junk; 
	char *commands[] = {
		"10",
			"Get Encoder values",
			"Set PWM",
			"Set Axis",
			"Get Odometry",
			"Set Velocity",
			"Reset Odometry",
			"Test CLoop",
			"Set Pos Trap",
			"Set Prop Vel Mode",
			"Quit"
	};


	init_serial();

	synchronize_with_robot();
	disable_robot_serial_timeouts();


	while(!quit_flag)
	{
		switch(DoMenu(commands)) {
		case MENU_GET_ENCODER:
			test_encoder_manual(axis);
			break;
			
		case MENU_SET_PWM:
			test_encoder_pwm(axis);
			break;

		case MENU_SET_AXIS:
			do {
				clrscr();
				printf("Select axis number : ");
				scanf("%d", &axis);
			} while((axis < 0) || (axis >= NUMBER_OF_AXES));
			break;

		case MENU_GET_ODOMETRY:
			get_odometry_keyboard();
			break;
		
		case MENU_SET_VELOCITY:
			set_robot_velocity_keyboard();
			break;

		case MENU_RESET_ODOMETRY:
			send_char_serial(SERIAL_RESET_ODOMETRY);
			receive_char_serial(&junk);
			break;

		case MENU_TEST_CLOOP:
			test_odo_closed_loop();
			break;
		

		case MENU_SET_POS_TRAP:
			set_trap_pos();
			break;

		case MENU_SET_PROP_VEL_MODE:
			send_char_serial(SERIAL_SET_PROP_VEL_MODE);
			receive_char_serial(&junk);
			break;

		case MENU_QUIT:
			quit_flag = 1;
			break;
			
		default:
			break;
		}
	}


	enable_robot_serial_timeouts();
	printf("Press any key to quit\n");
	return 0;
}

