#include <windows.h>

//	ASCII character codes to be used for Software flow control
//	Software flow control can be set using the SetFlowControl
//	function with FC_XONXOFF as the input
#define ASCII_XON       0x11
#define ASCII_XOFF      0x13

// Flow control flags
#define FC_NONE			0x00
#define FC_DTRDSR       0x01
#define FC_RTSCTS       0x02
#define FC_XONXOFF      0x04

#define PRINT_DEBUG_INFO	0	//	If this value is one debug information will be printed

// Buffer sizes of internal buffer of driver
#define SERIAL_RECEIVE_BUFFER_SIZE	4096
#define SERIAL_TRANSMIT_BUFFER_SIZE	4096

//	Comm port parameter structure
typedef struct tagSERIAL_CONNECTION
{
	HANDLE	idComDev			;	//	Handle to serial port
	int		iComPortNumber		;	//	Serial port number
	DWORD	dwReceiveBufferSize	;	//	Required receive buffer size of driver
	DWORD	dwTransmitBufferSize;	//	Required transmit buffer size of driver
	BYTE	bByteSize			;	//	Byte Size range(4-8)
	BYTE	bFlowCtrl			;	//	Flow Control
	BYTE	bParity				;	//	Parity
	BYTE	bStopBits			;	//	Stop Bits
	DWORD	dwBaudRate			;	//	Baud rate
	DWORD	dwTimeOutMultiplier	;	//	This multiplied by the number of bytes will
									//	be the timeout in milliseconds for read or write operations
} SERIAL_CONNECTION, *PSERIAL_CONNECTION ;

//	bFlowCtrl can be a combination of FC_NONE, FC_DTRDSR, FC_RTSCTS, FC_XONXOFF

//	bParity an be only one of NOPARITY, EVENPARITY, ODDPARITY, MARKPARITY, SPACEPARITY

//	bStopBits can be	ONESTOPBIT, ONE5STOPBITS, TWOSTOPBITS

//	dwBaudRate can be only one of the following:
//	CBR_110			CBR_300			CBR_600			CBR_1200		CBR_2400
//	CBR_4800		CBR_9600		CBR_14400		CBR_19200		CBR_38400,
//	CBR_56000		CBR_57600		CBR_115200		CBR_128000		CBR_256000


void BuildDefaultSerialConnectionStruct(PSERIAL_CONNECTION pSerialConnection);
//---------------------------------------------------------------------------
//	Description:
//		This routines Builds a default serial connection structure
//	Inputs:
//		PSERIAL_CONNECTION
//			Pointer of the structure of comm port parameters
//
//	The default serial settings are as follows
//
//	iComPortNumber			= 1;			//	Serial port number
//	dwReceiveBufferSize		= 4096;			//	Required receive buffer size of driver
//	dwTransmitBufferSize	= 4096;			//	Required transmit buffer size of driver
//	bByteSize				= 8;			//	Byte Size range(4-8)
//	bFlowCtrl				= FC_NONE;		//	Flow Control
//	bParity					= NOPARITY;		//	Parity
//	bStopBits				= ONESTOPBIT;	//	Stop Bits
//	dwBaudRate				= CBR_9600;		//	Baud rate
//	dwTimeOutMultiplier		= 1;			//	This multiplied by the number of bytes will
//
//---------------------------------------------------------------------------


BOOL OpenComPort( PSERIAL_CONNECTION pSerialConnection );
//---------------------------------------------------------------------------
//	Description:
//		This routines tries to open a connection to the comm
//		port and applies the settings
//	Inputs:
//		PSERIAL_CONNECTION
//			Pointer of the structure of comm port parameters
//	Outputs:
//		True if operation succsessful otherwise false
//
//---------------------------------------------------------------------------



BOOL ApplySerialSettings(PSERIAL_CONNECTION pSerialConnection);
//---------------------------------------------------------------------------
//	Description:
//		This routines sets up the DCB based on settings 
//		and performs a SetCommState().
//		This function needs to be called only if settings
//		are changed after OpenCommPort function call.
//	Inputs:
//		PSERIAL_CONNECTION
//			Pointer of the structure of comm port parameters
//	Outputs:
//		True if operation succsessful otherwise false
//
//---------------------------------------------------------------------------



DWORD ReadCommBlock( PSERIAL_CONNECTION pSerialConnection, LPSTR lpszBlock, DWORD dwBytesToBeRead );
//---------------------------------------------------------------------------
//	Description:
//		Reads a block from the COM port and stuffs it into
//		the provided buffer.
//
//	Inputs:
//		PSERIAL_CONNECTION
//			Pointer of the structure of comm port parameters
//		OVERLAPPED  pSerialConnection->osRead
//			Structure used for overlapped read
//		LPSTR lpszBlock
//			block used for storage of read bytes
//		DWORD dwBytesToBeRead
//			Number of bytes to be read
//	Outputs:
//		Number of bytes of data received from the comm port
//
//---------------------------------------------------------------------------



DWORD WriteCommBlock( PSERIAL_CONNECTION pSerialConnection, LPSTR lpByte , DWORD dwBytesToBeWritten );
//---------------------------------------------------------------------------
//  Writes a block of data to the COM port
//	Inputs:
//		PSERIAL_CONNECTION
//			Pointer of the structure of comm port parameters
//		LPSTR lpByte
//			pointer to data to write to port
//		DWORD dwBytesToBeWritten
//			Number of bytes to be written
//	Outputs:
//		Number of bytes of data written to the comm port
//
//---------------------------------------------------------------------------



char ReadChar(PSERIAL_CONNECTION pSerialConnection);
//---------------------------------------------------------------------------
//	Reads a character from the port.
//	Inputs:
//		PSERIAL_CONNECTION
//			Pointer of the structure of comm port parameters
//	Outputs:
//		Character received from the comm port
//---------------------------------------------------------------------------



void WriteChar(PSERIAL_CONNECTION pSerialConnection, char cInputChar);
//---------------------------------------------------------------------------
//	Writes a character to the port.
//	Inputs:
//		PSERIAL_CONNECTION
//			Pointer of the structure of comm port parameters
//		CInputChar
//			Character to be sent to the port
//---------------------------------------------------------------------------



void CloseComPort(PSERIAL_CONNECTION pSerialConnection);
//---------------------------------------------------------------------------
//	Closes the connection to the port.
//	Inputs:
//		PSERIAL_CONNECTION
//			Pointer of the structure of comm port parameters
//---------------------------------------------------------------------------